<?php


namespace Fastrack\Newsletter;

use Fastrack\Utilities\Slug;
use SilverStripe\Forms\GridField\GridFieldExportButton;
use \SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;

/**
 * @method Subscribers()
 */
class Newsletter extends DataObject implements PermissionProvider
{

	private $permissionCode = 'MANAGE_NEWSLETTERS';

	private static $table_name = 'Newsletter';

	private static $db = [
		'Name' => 'Varchar(255)',
		'Key' => 'Varchar(255)',
	];

	private static $has_many = [
		'Subscribers' => Subscriber::class,
	];

	private static $indexes = [
		'Key' => true,
	];

	public function getSubscriber($email)
	{
		return $this->Subscribers()->find('Email', $email);
	}

	public function onBeforeWrite() {
		$this->Key = Slug::generateSlug($this->Name, $this->Key);
		parent::onBeforeWrite();
	}

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$subscribers = $fields->fieldByName('Root.Subscribers.Subscribers');
		if ($subscribers) {
			$subscribers->getConfig()->addComponent(new GridFieldExportButton('before'));
		}
		return $fields;
	}

	public function providePermissions()
	{
		$permissions = [
			$this->permissionCode => [
				'name' => 'Manage newsletters',
				'category' => 'Newsletters',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		return Permission::check([$this->permissionCode]);
	}

	public function canEdit($member = null)
	{
		return Permission::check([$this->permissionCode]);
	}

	public function canDelete($member = null)
	{
		return Permission::check([$this->permissionCode]);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check([$this->permissionCode]);
	}

}
