<?php


namespace Fastrack\Newsletter;


use Fastrack\Utilities\FlashMessage;
use Fastrack\Utilities\GoogleReCaptcha;
use SilverStripe\Control\Email\Email;
use SilverStripe\Control\HTTPRequest;
use SilverStripe\ORM\ArrayList;
use SilverStripe\View\ArrayData;

class NewsletterController extends \PageController
{

	private static $allowed_actions = [
		'subscribe',
	];

	private static $url_handlers = [
		'subscribe' => 'subscribe',
	];


	private const NEWSLETTER_KEY = 'newsletter';
	private const FIRST_NAME = 'first_name';
	private const LAST_NAME = 'last_name';
	private const FULL_NAME = 'name';
	private const EMAIL = 'email';
	private const PHONE = 'phone';
	private const RECAPTCHA = 'g-recaptcha-response';
	private const NOTIFICATION_EMAIL = 'notification_email';

	public function subscribe(HTTPRequest $request)
	{
		$response = [
			'success' => true,
			'error' => '',
		];
		try {

			if (!GoogleReCaptcha::validate($this->getRequest()->requestVar(self::RECAPTCHA))) {
				throw new \Exception('Invalid reCAPTCHA validation');
			}

			$newsletterKey = $request->requestVar(self::NEWSLETTER_KEY);
			$newsletter = Newsletter::get()->find('Key', $newsletterKey);
			if (!$newsletter) {
				throw new \Exception('Could not find newsletter to subscribe to');
			}

			$email = $request->requestVar(self::EMAIL);
			if (!$email || !Email::is_valid_address($email)) {
				throw new \Exception('Invalid email address');
			}

			if ($newsletter->getSubscriber($email)) {
				throw new \Exception('You are already subscribed');
			}

			$subscriber = new Subscriber();
			$subscriber->FirstName = $request->requestVar(self::FIRST_NAME);
			$subscriber->LastName = $request->requestVar(self::LAST_NAME);
			$subscriber->FullName = $request->requestVar(self::FULL_NAME);
			$subscriber->Phone = $request->requestVar(self::PHONE);
			$subscriber->Email = $email;
			$subscriber->write();
			$newsletter->Subscribers()->add($subscriber);
			if ($this->getRequest()->requestVar(self::NOTIFICATION_EMAIL)) {
				$this->sendNotificationEmail();
			}
			$response['message'] = "Thank you {$this->getName()} for subscribing!";
		} catch (\Exception $e) {
			$this->getResponse()->setStatusCode(400);
			$response['success'] = false;
			$response['message'] = $e->getMessage();
		}
		$this->getResponse()->setBody(json_encode($response));
		$this->getResponse()->addHeader("Content-type", "application/json");
		return $this->getResponse();
	}


	private function sendNotificationEmail()
	{
		$email = new Email();
		$email->setHTMLTemplate('Fastrack\\Utilities\\ContactEmail');
		$email->setData($this->getEnquiryData());
		$email->setSubject('Newsletter subscription from website');
		$email->setTo(explode(',', $this->getRequest()->requestVar(self::NOTIFICATION_EMAIL)));
		$email->send();
	}

	private function getEnquiryData()
	{
		$data = new ArrayList([]);
		$fields = [self::FIRST_NAME, self::LAST_NAME, self::FULL_NAME, self::PHONE, self::EMAIL];
		foreach ($fields as $field) {
			$data->add(new ArrayData([
				'Field' => $field,
				'Value' => $this->getRequest()->requestVar($field)
			]));
		}
		return new ArrayData(['Fields' => $data]);
	}

	private function getName()
	{
		return $this->getRequest()->requestVar(self::FIRST_NAME) ?: $this->getRequest()->requestVar(self::FULL_NAME);
	}


}
