<?php


namespace Fastrack\Newsletter;


use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;

class Subscriber extends DataObject implements PermissionProvider
{

	private $permissionCode = 'MANAGE_SUBSCRIBERS';

	private static $table_name = 'Subscriber';

	private static $db = [
		'FirstName' => 'Varchar(255)',
		'LastName' => 'Varchar(255)',
		'FullName' => 'Varchar(255)',
		'Email' => 'Varchar(255)',
		'Phone' => 'Varchar(255)',
	];

	private static $has_one = [
		'Newsletter' => Newsletter::class,
	];

	private static $summary_fields = [
		'FullName',
		'Email',
	];

	private static $indexes = [
		'Email' => true,
	];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->removeByName('NewsletterID');
		return $fields;
	}

	public function providePermissions()
	{
		$permissions = [
			$this->permissionCode => [
				'name' => 'Manage subscribers',
				'category' => 'Newsletters',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		return Permission::check([$this->permissionCode]);
	}

	public function canEdit($member = null)
	{
		return Permission::check([$this->permissionCode]);
	}

	public function canDelete($member = null)
	{
		return Permission::check([$this->permissionCode]);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check([$this->permissionCode]);
	}

}
